@@ -57,7 +57,7 @@ BOOL WINAPI BackupEventLogA( HANDLE hEventLog, LPCSTR lpBackupFileName )
     LPWSTR backupW;
     BOOL ret;
 
-    backupW = SERV_dup(lpBackupFileName);
+    backupW = strdupAW(lpBackupFileName);
     ret = BackupEventLogW(hEventLog, backupW);
     heap_free(backupW);
 
@@ -114,7 +114,7 @@ BOOL WINAPI ClearEventLogA( HANDLE hEventLog, LPCSTR lpBackupFileName )
     LPWSTR backupW;
     BOOL ret;
 
-    backupW = SERV_dup(lpBackupFileName);
+    backupW = strdupAW(lpBackupFileName);
     ret = ClearEventLogW(hEventLog, backupW);
     heap_free(backupW);
 
@@ -164,30 +164,6 @@ BOOL WINAPI CloseEventLog( HANDLE hEventLog )
     return TRUE;
 }
 
-/******************************************************************************
- * ControlTraceW [ADVAPI32.@]
- *
- * Control a givel event trace session
- *
- */
-ULONG WINAPI ControlTraceW( TRACEHANDLE hSession, LPCWSTR SessionName, PEVENT_TRACE_PROPERTIES Properties, ULONG control )
-{
-    FIXME("(%s, %s, %p, %d) stub\n", wine_dbgstr_longlong(hSession), debugstr_w(SessionName), Properties, control);
-    return ERROR_SUCCESS;
-}
-
-/******************************************************************************
- * ControlTraceA [ADVAPI32.@]
- *
- * See ControlTraceW.
- *
- */
-ULONG WINAPI ControlTraceA( TRACEHANDLE hSession, LPCSTR SessionName, PEVENT_TRACE_PROPERTIES Properties, ULONG control )
-{
-    FIXME("(%s, %s, %p, %d) stub\n", wine_dbgstr_longlong(hSession), debugstr_a(SessionName), Properties, control);
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * FlushTraceA [ADVAPI32.@]
  */
@@ -238,20 +214,6 @@ ULONG WINAPI EnableTraceEx( LPCGUID provider, LPCGUID source, TRACEHANDLE hSessi
     return ERROR_SUCCESS;
 }
 
-/******************************************************************************
- * EnableTraceEx2 [ADVAPI32.@]
- */
-ULONG WINAPI EnableTraceEx2( TRACEHANDLE handle, LPCGUID provider, ULONG control, UCHAR level,
-                             ULONGLONG match_any, ULONGLONG match_all, ULONG timeout,
-                             PENABLE_TRACE_PARAMETERS params )
-{
-    FIXME("(%s, %s, %u, %u, %s, %s, %u, %p): stub\n", wine_dbgstr_longlong(handle),
-          debugstr_guid(provider), control, level, wine_dbgstr_longlong(match_any),
-          wine_dbgstr_longlong(match_all), timeout, params);
-
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * EnableTrace [ADVAPI32.@]
  */
@@ -430,8 +392,8 @@ HANDLE WINAPI OpenBackupEventLogA( LPCSTR lpUNCServerName, LPCSTR lpFileName )
     LPWSTR uncnameW, filenameW;
     HANDLE handle;
 
-    uncnameW = SERV_dup(lpUNCServerName);
-    filenameW = SERV_dup(lpFileName);
+    uncnameW = strdupAW(lpUNCServerName);
+    filenameW = strdupAW(lpFileName);
     handle = OpenBackupEventLogW(uncnameW, filenameW);
     heap_free(uncnameW);
     heap_free(filenameW);
@@ -489,8 +451,8 @@ HANDLE WINAPI OpenEventLogA( LPCSTR uncname, LPCSTR source )
     LPWSTR uncnameW, sourceW;
     HANDLE handle;
 
-    uncnameW = SERV_dup(uncname);
-    sourceW = SERV_dup(source);
+    uncnameW = strdupAW(uncname);
+    sourceW = strdupAW(source);
     handle = OpenEventLogW(uncnameW, sourceW);
     heap_free(uncnameW);
     heap_free(sourceW);
@@ -523,33 +485,6 @@ HANDLE WINAPI OpenEventLogW( LPCWSTR uncname, LPCWSTR source )
     return (HANDLE)0xcafe4242;
 }
 
-/******************************************************************************
- * QueryAllTracesW [ADVAPI32.@]
- *
- * Query information for started event trace sessions
- *
- */
-ULONG WINAPI QueryAllTracesW( PEVENT_TRACE_PROPERTIES * parray, ULONG arraycount, PULONG psessioncount )
-{
-    FIXME("(%p, %d, %p) stub\n", parray, arraycount, psessioncount);
-
-    if (psessioncount) *psessioncount = 0;
-    return ERROR_SUCCESS;
-}
-
-/******************************************************************************
- * QueryAllTracesA [ADVAPI32.@]
- *
- * See QueryAllTracesW.
- */
-ULONG WINAPI QueryAllTracesA( PEVENT_TRACE_PROPERTIES * parray, ULONG arraycount, PULONG psessioncount )
-{
-    FIXME("(%p, %d, %p) stub\n", parray, arraycount, psessioncount);
-
-    if (psessioncount) *psessioncount = 0;
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * ReadEventLogA [ADVAPI32.@]
  *
@@ -727,44 +662,6 @@ BOOL WINAPI ReportEventW( HANDLE hEventLog, WORD wType, WORD wCategory, DWORD dw
     return TRUE;
 }
 
-/******************************************************************************
- * StartTraceW [ADVAPI32.@]
- *
- * Register and start an event trace session
- *
- */
-ULONG WINAPI StartTraceW( PTRACEHANDLE pSessionHandle, LPCWSTR SessionName, PEVENT_TRACE_PROPERTIES Properties )
-{
-    FIXME("(%p, %s, %p) stub\n", pSessionHandle, debugstr_w(SessionName), Properties);
-    if (pSessionHandle) *pSessionHandle = 0xcafe4242;
-    return ERROR_SUCCESS;
-}
-
-/******************************************************************************
- * StartTraceA [ADVAPI32.@]
- *
- * See StartTraceW.
- *
- */
-ULONG WINAPI StartTraceA( PTRACEHANDLE pSessionHandle, LPCSTR SessionName, PEVENT_TRACE_PROPERTIES Properties )
-{
-    FIXME("(%p, %s, %p) stub\n", pSessionHandle, debugstr_a(SessionName), Properties);
-    if (pSessionHandle) *pSessionHandle = 0xcafe4242;
-    return ERROR_SUCCESS;
-}
-
-/******************************************************************************
- * StopTraceW [ADVAPI32.@]
- *
- * Stop an event trace session
- *
- */
-ULONG WINAPI StopTraceW( TRACEHANDLE session, LPCWSTR session_name, PEVENT_TRACE_PROPERTIES properties )
-{
-    FIXME("(%s, %s, %p) stub\n", wine_dbgstr_longlong(session), debugstr_w(session_name), properties);
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * StopTraceA [ADVAPI32.@]
  *
@@ -798,36 +695,6 @@ TRACEHANDLE WINAPI OpenTraceA( PEVENT_TRACE_LOGFILEA logfile )
     return INVALID_PROCESSTRACE_HANDLE;
 }
 
-/******************************************************************************
- * OpenTraceW [ADVAPI32.@]
- */
-TRACEHANDLE WINAPI OpenTraceW( PEVENT_TRACE_LOGFILEW logfile )
-{
-    static int once;
-
-    if (!once++) FIXME("%p: stub\n", logfile);
-    SetLastError(ERROR_ACCESS_DENIED);
-    return INVALID_PROCESSTRACE_HANDLE;
-}
-
-/******************************************************************************
- * ProcessTrace [ADVAPI32.@]
- */
-ULONG WINAPI ProcessTrace( PTRACEHANDLE HandleArray, ULONG HandleCount, LPFILETIME StartTime, LPFILETIME EndTime)
-{
-    FIXME("%p %u %p %p: stub\n", HandleArray, HandleCount, StartTime, EndTime);
-    return ERROR_CALL_NOT_IMPLEMENTED;
-}
-
-/******************************************************************************
- * CloseTrace [ADVAPI32.@]
- */
-ULONG WINAPI CloseTrace( TRACEHANDLE handle )
-{
-    FIXME("%s: stub\n", wine_dbgstr_longlong(handle));
-    return ERROR_INVALID_HANDLE;
-}
-
 /******************************************************************************
  * EnumerateTraceGuids [ADVAPI32.@]
  */
@@ -837,12 +704,3 @@ ULONG WINAPI EnumerateTraceGuids(PTRACE_GUID_PROPERTIES *propertiesarray,
     FIXME("%p %d %p: stub\n", propertiesarray, arraycount, guidcount);
     return ERROR_INVALID_PARAMETER;
 }
-
-/******************************************************************************
- * TraceSetInformation [ADVAPI32.@]
- */
-ULONG WINAPI TraceSetInformation(TRACEHANDLE handle, TRACE_INFO_CLASS infoclass, VOID* info, ULONG len)
-{
-    FIXME("%s %d %p %d: stub\n", wine_dbgstr_longlong(handle), infoclass, info, len);
-    return ERROR_CALL_NOT_IMPLEMENTED;
-}
