@@ -25,6 +25,7 @@
 #include "config.h"
 #include "wine/port.h"
 
+#include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -56,6 +57,7 @@ char* CDECL MSVCRT__strdup(const char* str)
  */
 int CDECL MSVCRT__strlwr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t locale)
 {
+    MSVCRT_pthreadlocinfo locinfo;
     char *ptr = str;
 
     if (!str || !len)
@@ -77,10 +79,27 @@ int CDECL MSVCRT__strlwr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t loca
         return MSVCRT_EINVAL;
     }
 
-    while (*str)
+    if(!locale)
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
+
+    if(!locinfo->lc_handle[MSVCRT_LC_CTYPE])
     {
-        *str = MSVCRT__tolower_l((unsigned char)*str, locale);
-        str++;
+        while (*str)
+        {
+            if (*str >= 'A' && *str <= 'Z')
+                *str -= 'A' - 'a';
+            str++;
+        }
+    }
+    else
+    {
+        while (*str)
+        {
+            *str = MSVCRT__tolower_l((unsigned char)*str, locale);
+            str++;
+        }
     }
 
     return 0;
@@ -325,8 +344,6 @@ void CDECL MSVCRT__swab(char* src, char* dst, int len)
   }
 }
 
-#if _MSVCR_VER >= 140
-
 enum round {
     ROUND_ZERO, /* only used when dropped part contains only zeros */
     ROUND_DOWN,
@@ -426,6 +443,8 @@ static double make_double(int sign, int exp, ULONGLONG m, enum round round, int
     return *((double*)&bits);
 }
 
+#if _MSVCR_VER >= 140
+
 static inline int hex2int(char c)
 {
     if (c >= '0' && c <= '9')
@@ -437,28 +456,30 @@ static inline int hex2int(char c)
     return -1;
 }
 
-static double strtod16(int sign, const char *p, char **end,
-        MSVCRT_pthreadlocinfo locinfo, int *err)
+static double strtod16(MSVCRT_wchar_t get(void *ctx), void unget(void *ctx),
+        void *ctx, int sign, MSVCRT_pthreadlocinfo locinfo, int *err)
 {
+    BOOL found_digit = FALSE, found_dp = FALSE;
     enum round round = ROUND_ZERO;
-    BOOL found_digit = FALSE;
+    MSVCRT_wchar_t nch;
     ULONGLONG m = 0;
     int val, exp = 0;
 
+    nch = get(ctx);
     while(m < MSVCRT_UI64_MAX/16)
     {
-        val = hex2int(*p);
+        val = hex2int(nch);
         if (val == -1) break;
         found_digit = TRUE;
-        p++;
+        nch = get(ctx);
 
         m = m*16 + val;
     }
     while(1)
     {
-        val = hex2int(*p);
+        val = hex2int(nch);
         if (val == -1) break;
-        p++;
+        nch = get(ctx);
         exp += 4;
 
         if (val || round != ROUND_ZERO)
@@ -469,29 +490,33 @@ static double strtod16(int sign, const char *p, char **end,
         }
     }
 
-    if(*p == *locinfo->lconv->decimal_point)
-        p++;
+    if(nch == *locinfo->lconv->decimal_point)
+    {
+        found_dp = TRUE;
+        nch = get(ctx);
+    }
     else if (!found_digit)
     {
-        if(end) *end = (char*)p - 1;
+        if(nch!=MSVCRT_WEOF) unget(ctx);
+        unget(ctx);
         return 0.0;
     }
 
     while(m <= MSVCRT_UI64_MAX/16)
     {
-        val = hex2int(*p);
+        val = hex2int(nch);
         if (val == -1) break;
         found_digit = TRUE;
-        p++;
+        nch = get(ctx);
 
         m = m*16 + val;
         exp -= 4;
     }
     while(1)
     {
-        val = hex2int(*p);
+        val = hex2int(nch);
         if (val == -1) break;
-        p++;
+        nch = get(ctx);
 
         if (val || round != ROUND_ZERO)
         {
@@ -503,63 +528,407 @@ static double strtod16(int sign, const char *p, char **end,
 
     if (!found_digit)
     {
-        if(end) *end = (char*)p - 2;
+        if (nch != MSVCRT_WEOF) unget(ctx);
+        if (found_dp) unget(ctx);
+        unget(ctx);
         return 0.0;
     }
 
-    if(*p=='p' || *p=='P') {
+    if(nch=='p' || nch=='P') {
+        BOOL found_sign = FALSE;
         int e=0, s=1;
 
-        p++;
-        if(*p == '-') {
+        nch = get(ctx);
+        if(nch == '-') {
+            found_sign = TRUE;
             s = -1;
-            p++;
-        } else if(*p == '+')
-            p++;
-
-        if(*p>='0' && *p<='9') {
-            while(*p>='0' && *p<='9') {
-                if(e>INT_MAX/10 || (e=e*10+*p-'0')<0)
+            nch = get(ctx);
+        } else if(nch == '+') {
+            found_sign = TRUE;
+            nch = get(ctx);
+        }
+        if(nch>='0' && nch<='9') {
+            while(nch>='0' && nch<='9') {
+                if(e>INT_MAX/10 || e*10>INT_MAX-nch+'0')
                     e = INT_MAX;
-                p++;
+                else
+                    e = e*10+nch-'0';
+                nch = get(ctx);
             }
+            if((nch!=MSVCRT_WEOF) && (nch < '0' || nch > '9')) unget(ctx);
             e *= s;
 
-            if(exp<0 && e<0 && exp+e>=0) exp = INT_MIN;
-            else if(exp>0 && e>0 && exp+e<0) exp = INT_MAX;
+            if(e<0 && exp<INT_MIN-e) exp = INT_MIN;
+            else if(e>0 && exp>INT_MAX-e) exp = INT_MAX;
             else exp += e;
         } else {
-            if(*p=='-' || *p=='+')
-                p--;
-            p--;
+            if(nch != MSVCRT_WEOF) unget(ctx);
+            if(found_sign) unget(ctx);
+            unget(ctx);
         }
     }
 
-    if (end) *end = (char*)p;
     return make_double(sign, exp, m, round, err);
 }
 #endif
 
-static double strtod_helper(const char *str, char **end, MSVCRT__locale_t locale, int *err)
+#define LIMB_DIGITS 9           /* each DWORD stores up to 9 digits */
+#define LIMB_MAX 1000000000     /* 10^9 */
+#define BNUM_IDX(i) ((i) & 127)
+/* bnum represents real number with fixed decimal point (after 2 limbs) */
+struct bnum {
+    DWORD data[128]; /* circular buffer, base 10 number */
+    int b; /* least significant digit position */
+    int e; /* most significant digit position + 1 */
+};
+
+/* Returns integral part of bnum */
+static inline ULONGLONG bnum_to_mant(struct bnum *b)
+{
+    ULONGLONG ret = (ULONGLONG)b->data[BNUM_IDX(b->e-1)] * LIMB_MAX;
+    if(b->b != b->e-1) ret += b->data[BNUM_IDX(b->e-2)];
+    return ret;
+}
+
+/* Returns TRUE if new most significant limb was added */
+static inline BOOL bnum_lshift(struct bnum *b, int shift)
+{
+    DWORD rest = 0;
+    ULONGLONG tmp;
+    int i;
+
+    /* The limbs number can change by up to 1 so shift <= 29 */
+    assert(shift <= 29);
+
+    for(i=b->b; i<b->e; i++) {
+        tmp = ((ULONGLONG)b->data[BNUM_IDX(i)] << shift) + rest;
+        rest = tmp / LIMB_MAX;
+        b->data[BNUM_IDX(i)] = tmp % LIMB_MAX;
+
+        if(i == b->b && !b->data[BNUM_IDX(i)])
+            b->b++;
+    }
+
+    if(rest) {
+        b->data[BNUM_IDX(b->e)] = rest;
+        b->e++;
+
+        if(BNUM_IDX(b->b) == BNUM_IDX(b->e)) {
+            if(b->data[BNUM_IDX(b->b)]) b->data[BNUM_IDX(b->b+1)] |= 1;
+            b->b++;
+        }
+        return TRUE;
+    }
+    return FALSE;
+}
+
+/* Returns TRUE if most significant limb was removed */
+static inline BOOL bnum_rshift(struct bnum *b, int shift)
+{
+    DWORD tmp, rest = 0;
+    BOOL ret = FALSE;
+    int i;
+
+    /* Compute LIMB_MAX << shift without accuracy loss */
+    assert(shift <= 9);
+
+    for(i=b->e-1; i>=b->b; i--) {
+        tmp = b->data[BNUM_IDX(i)] & ((1<<shift)-1);
+        b->data[BNUM_IDX(i)] = (b->data[BNUM_IDX(i)] >> shift) + rest;
+        rest = (LIMB_MAX >> shift) * tmp;
+        if(i==b->e-1 && !b->data[BNUM_IDX(i)]) {
+            b->e--;
+            ret = TRUE;
+        }
+    }
+
+    if(rest) {
+        if(BNUM_IDX(b->b-1) == BNUM_IDX(b->e)) {
+            if(rest) b->data[BNUM_IDX(b->b)] |= 1;
+        } else {
+            b->b--;
+            b->data[BNUM_IDX(b->b)] = rest;
+        }
+    }
+    return ret;
+}
+
+static inline void bnum_mult(struct bnum *b, int mult)
+{
+    DWORD rest = 0;
+    ULONGLONG tmp;
+    int i;
+
+    assert(mult <= LIMB_MAX);
+
+    for(i=b->b; i<b->e; i++) {
+        tmp = ((ULONGLONG)b->data[BNUM_IDX(i)] * mult) + rest;
+        rest = tmp / LIMB_MAX;
+        b->data[BNUM_IDX(i)] = tmp % LIMB_MAX;
+
+        if(i == b->b && !b->data[BNUM_IDX(i)])
+            b->b++;
+    }
+
+    if(rest) {
+        b->data[BNUM_IDX(b->e)] = rest;
+        b->e++;
+
+        if(BNUM_IDX(b->b) == BNUM_IDX(b->e)) {
+            if(b->data[BNUM_IDX(b->b)]) b->data[BNUM_IDX(b->b+1)] |= 1;
+            b->b++;
+        }
+    }
+}
+
+double parse_double(MSVCRT_wchar_t (*get)(void *ctx), void (*unget)(void *ctx),
+        void *ctx, MSVCRT_pthreadlocinfo locinfo, int *err)
+{
+    static const int p10s[] = { 10, 100, 1000, 10000, 100000, 1000000, 10000000, 100000000 };
+
+#if _MSVCR_VER >= 140
+    MSVCRT_wchar_t _infinity[] = { 'i', 'n', 'f', 'i', 'n', 'i', 't', 'y', 0 };
+    MSVCRT_wchar_t _nan[] = { 'n', 'a', 'n', 0 };
+    MSVCRT_wchar_t *str_match = NULL;
+    int matched=0;
+#endif
+    BOOL found_digit = FALSE, found_dp = FALSE, found_sign = FALSE;
+    int e2 = 0, dp=0, sign=1, off, limb_digits = 0, i;
+    enum round round = ROUND_ZERO;
+    MSVCRT_wchar_t nch;
+    struct bnum b;
+
+    nch = get(ctx);
+    if(nch == '-') {
+        found_sign = TRUE;
+        sign = -1;
+        nch = get(ctx);
+    } else if(nch == '+') {
+        found_sign = TRUE;
+        nch = get(ctx);
+    }
+
+#if _MSVCR_VER >= 140
+    if(nch == _infinity[0] || nch == MSVCRT__toupper(_infinity[0]))
+        str_match = _infinity;
+    if(nch == _nan[0] || nch == MSVCRT__toupper(_nan[0]))
+        str_match = _nan;
+    while(str_match && nch != MSVCRT_WEOF &&
+            (nch == str_match[matched] || nch == MSVCRT__toupper(str_match[matched]))) {
+        nch = get(ctx);
+        matched++;
+    }
+    if(str_match) {
+        int keep = 0;
+        if(matched >= 8) keep = 8;
+        else if(matched >= 3) keep = 3;
+        if(nch != MSVCRT_WEOF) unget(ctx);
+        for (; matched > keep; matched--) {
+            unget(ctx);
+        }
+        if(keep) {
+            if (str_match == _infinity) return sign*INFINITY;
+            if (str_match == _nan) return sign*NAN;
+        } else if(found_sign) {
+            unget(ctx);
+        }
+
+        return 0.0;
+    }
+
+    if(nch == '0') {
+        found_digit = TRUE;
+        nch = get(ctx);
+        if(nch == 'x' || nch == 'X')
+            return strtod16(get, unget, ctx, sign, locinfo, err);
+    }
+#endif
+
+    while(nch == '0') {
+        found_digit = TRUE;
+        nch = get(ctx);
+    }
+
+    b.data[0] = 0;
+    b.b = 0;
+    b.e = 1;
+    while(nch>='0' && nch<='9') {
+        found_digit = TRUE;
+        if(limb_digits == LIMB_DIGITS) {
+            if(BNUM_IDX(b.b-1) == BNUM_IDX(b.e)) break;
+            else {
+                b.b--;
+                b.data[BNUM_IDX(b.b)] = 0;
+                limb_digits = 0;
+            }
+        }
+
+        b.data[BNUM_IDX(b.b)] = b.data[BNUM_IDX(b.b)] * 10 + nch - '0';
+        limb_digits++;
+        nch = get(ctx);
+        dp++;
+    }
+    while(nch>='0' && nch<='9') {
+        if(nch != '0') b.data[BNUM_IDX(b.b)] |= 1;
+        nch = get(ctx);
+        dp++;
+    }
+
+    if(nch == *locinfo->lconv->decimal_point) {
+        found_dp = TRUE;
+        nch = get(ctx);
+    }
+
+    /* skip leading '0' */
+    if(nch=='0' && !limb_digits && !b.b) {
+        found_digit = TRUE;
+        while(nch == '0') {
+            nch = get(ctx);
+            dp--;
+        }
+    }
+
+    while(nch>='0' && nch<='9') {
+        found_digit = TRUE;
+        if(limb_digits == LIMB_DIGITS) {
+            if(BNUM_IDX(b.b-1) == BNUM_IDX(b.e)) break;
+            else {
+                b.b--;
+                b.data[BNUM_IDX(b.b)] = 0;
+                limb_digits = 0;
+            }
+        }
+
+        b.data[BNUM_IDX(b.b)] = b.data[BNUM_IDX(b.b)] * 10 + nch - '0';
+        limb_digits++;
+        nch = get(ctx);
+    }
+    while(nch>='0' && nch<='9') {
+        if(nch != '0') b.data[BNUM_IDX(b.b)] |= 1;
+        nch = get(ctx);
+    }
+
+    if(!found_digit) {
+        if(nch != MSVCRT_WEOF) unget(ctx);
+        if(found_dp) unget(ctx);
+        if(found_sign) unget(ctx);
+        return 0.0;
+    }
+
+    if(nch=='e' || nch=='E' || nch=='d' || nch=='D') {
+        int e=0, s=1;
+
+        nch = get(ctx);
+        if(nch == '-') {
+            found_sign = TRUE;
+            s = -1;
+            nch = get(ctx);
+        } else if(nch == '+') {
+            found_sign = TRUE;
+            nch = get(ctx);
+        } else {
+            found_sign = FALSE;
+        }
+
+        if(nch>='0' && nch<='9') {
+            while(nch>='0' && nch<='9') {
+                if(e>INT_MAX/10 || e*10>INT_MAX-nch+'0')
+                    e = INT_MAX;
+                else
+                    e = e*10+nch-'0';
+                nch = get(ctx);
+            }
+            if(nch != MSVCRT_WEOF) unget(ctx);
+            e *= s;
+
+            if(e<0 && dp<INT_MIN-e) dp = INT_MIN;
+            else if(e>0 && dp>INT_MAX-e) dp = INT_MAX;
+            else dp += e;
+        } else {
+            if(nch != MSVCRT_WEOF) unget(ctx);
+            if(found_sign) unget(ctx);
+            unget(ctx);
+        }
+    } else if(nch != MSVCRT_WEOF) {
+        unget(ctx);
+    }
+
+    if(!b.data[BNUM_IDX(b.e-1)]) return make_double(sign, 0, 0, ROUND_ZERO, err);
+
+    /* Fill last limb with 0 if needed */
+    if(b.b+1 != b.e) {
+        for(; limb_digits != LIMB_DIGITS; limb_digits++)
+            b.data[BNUM_IDX(b.b)] *= 10;
+    }
+    for(; BNUM_IDX(b.b) < BNUM_IDX(b.e); b.b++) {
+        if(b.data[BNUM_IDX(b.b)]) break;
+    }
+
+    /* move decimal point to limb boundry */
+    if(limb_digits==dp && b.b==b.e-1)
+        return make_double(sign, 0, b.data[BNUM_IDX(b.e-1)], ROUND_ZERO, err);
+    off = (dp - limb_digits) % LIMB_DIGITS;
+    if(off < 0) off += LIMB_DIGITS;
+    if(off) bnum_mult(&b, p10s[off-1]);
+
+    if(!err) err = MSVCRT__errno();
+    if(dp-1 > MSVCRT_DBL_MAX_10_EXP)
+        return make_double(sign, INT_MAX, 1, ROUND_ZERO, err);
+    /* Count part of exponent stored in denormalized mantissa. */
+    /* Increase exponent range to handle subnormals. */
+    if(dp-1 < MSVCRT_DBL_MIN_10_EXP-MSVCRT_DBL_DIG-18)
+        return make_double(sign, INT_MIN, 1, ROUND_ZERO, err);
+
+    while(dp > 2*LIMB_DIGITS) {
+        if(bnum_rshift(&b, 9)) dp -= LIMB_DIGITS;
+        e2 += 9;
+    }
+    while(dp <= LIMB_DIGITS) {
+        if(bnum_lshift(&b, 29)) dp += LIMB_DIGITS;
+        e2 -= 29;
+    }
+    while(b.data[BNUM_IDX(b.e-1)] < LIMB_MAX/10) {
+        bnum_lshift(&b, 1);
+        e2--;
+    }
+
+    /* Check if fractional part is non-zero */
+    /* Caution: it's only correct because bnum_to_mant returns more then 53 bits */
+    for(i=b.e-3; i>=b.b; i--) {
+        if (!b.data[BNUM_IDX(b.b)]) continue;
+        round = ROUND_DOWN;
+        break;
+    }
+
+    return make_double(sign, e2, bnum_to_mant(&b), round, err);
+}
+
+static MSVCRT_wchar_t strtod_str_get(void *ctx)
+{
+    const char **p = ctx;
+    if (!**p) return MSVCRT_WEOF;
+    return *(*p)++;
+}
+
+static void strtod_str_unget(void *ctx)
+{
+    const char **p = ctx;
+    (*p)--;
+}
+
+static inline double strtod_helper(const char *str, char **end, MSVCRT__locale_t locale, int *err)
 {
     MSVCRT_pthreadlocinfo locinfo;
-    unsigned __int64 d=0, hlp;
-    unsigned fpcontrol;
-    int exp=0, sign=1;
-    const char *p;
+    const char *beg, *p;
     double ret;
-    long double lret=1, expcnt = 10;
-    BOOL found_digit = FALSE, negexp;
 
-    if(err)
-        *err = 0;
-    else if(!MSVCRT_CHECK_PMT(str != NULL)) {
-        if (end)
-            *end = NULL;
+    if (err) *err = 0;
+    if (!MSVCRT_CHECK_PMT(str != NULL)) {
+        if (end) *end = NULL;
         return 0;
     }
 
-    if(!locale)
+    if (!locale)
         locinfo = get_locinfo();
     else
         locinfo = locale->locinfo;
@@ -567,127 +936,10 @@ static double strtod_helper(const char *str, char **end, MSVCRT__locale_t locale
     p = str;
     while(MSVCRT__isspace_l((unsigned char)*p, locale))
         p++;
+    beg = p;
 
-    if(*p == '-') {
-        sign = -1;
-        p++;
-    } else  if(*p == '+')
-        p++;
-
-#if _MSVCR_VER >= 140
-    if(MSVCRT__tolower_l(p[0], locale) == 'i' && MSVCRT__tolower_l(p[1], locale) == 'n'
-            && MSVCRT__tolower_l(p[2], locale) == 'f') {
-        if(end)
-            *end = (char*) &p[3];
-        if(MSVCRT__tolower_l(p[3], locale) == 'i' && MSVCRT__tolower_l(p[4], locale) == 'n'
-            && MSVCRT__tolower_l(p[5], locale) == 'i' && MSVCRT__tolower_l(p[6], locale) == 't'
-            && MSVCRT__tolower_l(p[7], locale) == 'y' && end)
-            *end = (char*) &p[8];
-        return sign*INFINITY;
-    }
-    if(MSVCRT__tolower_l(p[0], locale) == 'n' &&
-       MSVCRT__tolower_l(p[1], locale) == 'a' &&
-       MSVCRT__tolower_l(p[2], locale) == 'n') {
-        if(end)
-            *end = (char*) &p[3];
-        return NAN;
-    }
-
-    if(p[0] == '0' && (p[1] == 'x' || p[1] == 'X')) {
-        p += 2;
-        return strtod16(sign, p, end, locinfo, err);
-    }
-#endif
-
-    while(*p>='0' && *p<='9') {
-        found_digit = TRUE;
-        hlp = d * 10 + *p++ - '0';
-        if(d>MSVCRT_UI64_MAX/10 || hlp<d) {
-            exp++;
-            break;
-        } else
-            d = hlp;
-    }
-    while(*p>='0' && *p<='9') {
-        exp++;
-        p++;
-    }
-
-    if(*p == *locinfo->lconv->decimal_point)
-        p++;
-
-    while(*p>='0' && *p<='9') {
-        found_digit = TRUE;
-        hlp = d * 10 + *p++ - '0';
-        if(d>MSVCRT_UI64_MAX/10 || hlp<d)
-            break;
-        d = hlp;
-        exp--;
-    }
-    while(*p>='0' && *p<='9')
-        p++;
-
-    if(!found_digit) {
-        if(end)
-            *end = (char*)str;
-        return 0.0;
-    }
-
-    if(*p=='e' || *p=='E' || *p=='d' || *p=='D') {
-        int e=0, s=1;
-
-        p++;
-        if(*p == '-') {
-            s = -1;
-            p++;
-        } else if(*p == '+')
-            p++;
-
-        if(*p>='0' && *p<='9') {
-            while(*p>='0' && *p<='9') {
-                if(e>INT_MAX/10 || (e=e*10+*p-'0')<0)
-                    e = INT_MAX;
-                p++;
-            }
-            e *= s;
-
-            if(exp<0 && e<0 && exp+e>=0) exp = INT_MIN;
-            else if(exp>0 && e>0 && exp+e<0) exp = INT_MAX;
-            else exp += e;
-        } else {
-            if(*p=='-' || *p=='+')
-                p--;
-            p--;
-        }
-    }
-
-    fpcontrol = _control87(0, 0);
-    _control87(MSVCRT__EM_DENORMAL|MSVCRT__EM_INVALID|MSVCRT__EM_ZERODIVIDE
-            |MSVCRT__EM_OVERFLOW|MSVCRT__EM_UNDERFLOW|MSVCRT__EM_INEXACT, 0xffffffff);
-
-    negexp = (exp < 0);
-    if(negexp)
-        exp = -exp;
-    while(exp) {
-        if(exp & 1)
-            lret *= expcnt;
-        exp /= 2;
-        expcnt = expcnt*expcnt;
-    }
-    ret = (long double)sign * (negexp ? d/lret : d*lret);
-
-    _control87(fpcontrol, 0xffffffff);
-
-    if((d && ret==0.0) || isinf(ret)) {
-        if(err)
-            *err = MSVCRT_ERANGE;
-        else
-            *MSVCRT__errno() = MSVCRT_ERANGE;
-    }
-
-    if(end)
-        *end = (char*)p;
-
+    ret = parse_double(strtod_str_get, strtod_str_unget, &p, locinfo, err);
+    if (end) *end = (p == beg ? (char*)str : (char*)p);
     return ret;
 }
 
@@ -2132,8 +2384,8 @@ void* __cdecl MSVCRT_memchr(const void *ptr, int c, MSVCRT_size_t n)
 int __cdecl MSVCRT_strcmp(const char *str1, const char *str2)
 {
     while (*str1 && *str1 == *str2) { str1++; str2++; }
-    if (*str1 > *str2) return 1;
-    if (*str1 < *str2) return -1;
+    if ((unsigned char)*str1 > (unsigned char)*str2) return 1;
+    if ((unsigned char)*str1 < (unsigned char)*str2) return -1;
     return 0;
 }
 
@@ -2144,9 +2396,7 @@ int __cdecl MSVCRT_strncmp(const char *str1, const char *str2, MSVCRT_size_t len
 {
     if (!len) return 0;
     while (--len && *str1 && *str1 == *str2) { str1++; str2++; }
-    if (*str1 > *str2) return 1;
-    if (*str1 < *str2) return -1;
-    return 0;
+    return (unsigned char)*str1 - (unsigned char)*str2;
 }
 
 /*********************************************************************
@@ -2155,6 +2405,7 @@ int __cdecl MSVCRT_strncmp(const char *str1, const char *str2, MSVCRT_size_t len
 int __cdecl MSVCRT__strnicmp_l(const char *s1, const char *s2,
         MSVCRT_size_t count, MSVCRT__locale_t locale)
 {
+    MSVCRT_pthreadlocinfo locinfo;
     int c1, c2;
 
     if(s1==NULL || s2==NULL)
@@ -2163,9 +2414,26 @@ int __cdecl MSVCRT__strnicmp_l(const char *s1, const char *s2,
     if(!count)
         return 0;
 
+    if(!locale)
+        locinfo = get_locinfo();
+    else
+        locinfo = locale->locinfo;
+
+    if(!locinfo->lc_handle[MSVCRT_LC_CTYPE])
+    {
+        do {
+            if ((c1 = *s1++) >= 'A' && c1 <= 'Z')
+                c1 -= 'A' - 'a';
+            if ((c2 = *s2++) >= 'A' && c2 <= 'Z')
+                c2 -= 'A' - 'a';
+        }while(--count && c1 && c1==c2);
+
+        return c1-c2;
+    }
+
     do {
-        c1 = MSVCRT__tolower_l(*s1++, locale);
-        c2 = MSVCRT__tolower_l(*s2++, locale);
+        c1 = MSVCRT__tolower_l((unsigned char)*s1++, locale);
+        c2 = MSVCRT__tolower_l((unsigned char)*s2++, locale);
     }while(--count && c1 && c1==c2);
 
     return c1-c2;
