@@ -726,7 +726,8 @@ static void *allocate_large_block( HEAP *heap, DWORD flags, SIZE_T size )
     LPVOID address = NULL;
 
     if (block_size < size) return NULL;  /* overflow */
-    if (virtual_alloc_aligned( &address, 0, &block_size, MEM_COMMIT, get_protection_type( flags ), 5 ))
+    if (NtAllocateVirtualMemory( NtCurrentProcess(), &address, 0, &block_size,
+                                 MEM_COMMIT, get_protection_type( flags )))
     {
         WARN("Could not allocate block for %08lx bytes\n", size );
         return NULL;
@@ -1508,14 +1509,9 @@ void heap_set_debug_flags( HANDLE handle )
     if ((heap->flags & HEAP_GROWABLE) && !heap->pending_free &&
         ((flags & HEAP_FREE_CHECKING_ENABLED) || RUNNING_ON_VALGRIND))
     {
-        void *ptr = NULL;
-        SIZE_T size = MAX_FREE_PENDING * sizeof(*heap->pending_free);
-
-        if (!virtual_alloc_aligned( &ptr, 0, &size, MEM_COMMIT, PAGE_READWRITE, 4 ))
-        {
-            heap->pending_free = ptr;
-            heap->pending_pos = 0;
-        }
+        heap->pending_free = RtlAllocateHeap( GetProcessHeap(), HEAP_ZERO_MEMORY,
+                                              MAX_FREE_PENDING * sizeof(*heap->pending_free) );
+        heap->pending_pos = 0;
     }
 }
 
@@ -1622,12 +1618,7 @@ HANDLE WINAPI RtlDestroyHeap( HANDLE heap )
         NtFreeVirtualMemory( NtCurrentProcess(), &addr, &size, MEM_RELEASE );
     }
     subheap_notify_free_all(&heapPtr->subheap);
-    if (heapPtr->pending_free)
-    {
-        size = 0;
-        addr = heapPtr->pending_free;
-        NtFreeVirtualMemory( NtCurrentProcess(), &addr, &size, MEM_RELEASE );
-    }
+    RtlFreeHeap( GetProcessHeap(), 0, heapPtr->pending_free );
     size = 0;
     addr = heapPtr->subheap.base;
     NtFreeVirtualMemory( NtCurrentProcess(), &addr, &size, MEM_RELEASE );
