@@ -93,10 +93,19 @@ static BOOL video_init(void)
 #endif
 }
 
+struct caps
+{
+    __u32 pixelformat;
+    AM_MEDIA_TYPE media_type;
+    VIDEOINFOHEADER video_info;
+    VIDEO_STREAM_CONFIG_CAPS config;
+};
+
 struct _Capture
 {
-    UINT width, height, bitDepth, fps, outputwidth, outputheight;
-    BOOL swresize;
+    const struct caps *current_caps;
+    struct caps *caps;
+    LONG caps_count;
 
     struct strmbase_source *pin;
     int fd, mmap;
@@ -116,136 +125,103 @@ static int xioctl(int fd, int request, void * arg)
     return r;
 }
 
-HRESULT qcap_driver_destroy(Capture *capBox)
+HRESULT qcap_driver_destroy(Capture *device)
 {
-    TRACE("%p\n", capBox);
+    if (device->fd != -1)
+        video_close(device->fd);
+    if (device->caps_count)
+        heap_free(device->caps);
+    heap_free(device);
 
-    if( capBox->fd != -1 )
-        video_close(capBox->fd);
-    CoTaskMemFree(capBox);
     return S_OK;
 }
 
+static const struct caps *find_caps(Capture *device, const AM_MEDIA_TYPE *mt)
+{
+    const VIDEOINFOHEADER *video_info = (VIDEOINFOHEADER *)mt->pbFormat;
+    LONG index;
+
+    if (mt->cbFormat < sizeof(VIDEOINFOHEADER) || !video_info)
+        return NULL;
+
+    for (index = 0; index < device->caps_count; index++)
+    {
+        struct caps *caps = &device->caps[index];
+
+        if (IsEqualGUID(&mt->formattype, &caps->media_type.formattype)
+                && video_info->bmiHeader.biWidth == caps->video_info.bmiHeader.biWidth
+                && video_info->bmiHeader.biHeight == caps->video_info.bmiHeader.biHeight)
+            return caps;
+    }
+    return NULL;
+}
+
 HRESULT qcap_driver_check_format(Capture *device, const AM_MEDIA_TYPE *mt)
 {
-    HRESULT hr;
     TRACE("device %p, mt %p.\n", device, mt);
 
     if (!mt)
         return E_POINTER;
 
     if (!IsEqualGUID(&mt->majortype, &MEDIATYPE_Video))
-        return S_FALSE;
+        return E_FAIL;
 
-    if (IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo) && mt->pbFormat
-            && mt->cbFormat >= sizeof(VIDEOINFOHEADER))
+    if (find_caps(device, mt))
+        return S_OK;
+
+    return E_FAIL;
+}
+
+static BOOL set_caps(Capture *device, const struct caps *caps)
+{
+    struct v4l2_format format = {0};
+    LONG width, height;
+
+    width = caps->video_info.bmiHeader.biWidth;
+    height = caps->video_info.bmiHeader.biHeight;
+
+    format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+    format.fmt.pix.pixelformat = caps->pixelformat;
+    format.fmt.pix.width = width;
+    format.fmt.pix.height = height;
+    if (xioctl(device->fd, VIDIOC_S_FMT, &format) == -1
+            || format.fmt.pix.pixelformat != caps->pixelformat
+            || format.fmt.pix.width != width
+            || format.fmt.pix.height != height)
     {
-        VIDEOINFOHEADER *vih = (VIDEOINFOHEADER *)mt->pbFormat;
-        if (vih->bmiHeader.biBitCount == 24 && vih->bmiHeader.biCompression == BI_RGB)
-            hr = S_OK;
-        else
-        {
-            FIXME("Unsupported compression %#x, bpp %u.\n", vih->bmiHeader.biCompression,
-                    vih->bmiHeader.biBitCount);
-            hr = S_FALSE;
-        }
+        ERR("Failed to set pixel format: %s.\n", strerror(errno));
+        return FALSE;
     }
-    else
-        hr = VFW_E_INVALIDMEDIATYPE;
 
-    return hr;
+    device->current_caps = caps;
+
+    return TRUE;
 }
 
 HRESULT qcap_driver_set_format(Capture *device, AM_MEDIA_TYPE *mt)
 {
-    struct v4l2_format format = {0};
-    int newheight, newwidth;
-    VIDEOINFOHEADER *vih;
-    int fd = device->fd;
-    HRESULT hr;
+    const struct caps *caps;
 
-    if (FAILED(hr = qcap_driver_check_format(device, mt)))
-        return hr;
-    vih = (VIDEOINFOHEADER *)mt->pbFormat;
+    caps = find_caps(device, mt);
+    if (!caps)
+        return E_FAIL;
 
-    newwidth = vih->bmiHeader.biWidth;
-    newheight = vih->bmiHeader.biHeight;
-
-    if (device->height == newheight && device->width == newwidth)
+    if (device->current_caps == caps)
         return S_OK;
 
-    format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-    if (xioctl(fd, VIDIOC_G_FMT, &format) == -1)
-    {
-        ERR("Failed to get current format: %s\n", strerror(errno));
+    if (!set_caps(device, caps))
         return VFW_E_TYPE_NOT_ACCEPTED;
-    }
 
-    format.fmt.pix.width = newwidth;
-    format.fmt.pix.height = newheight;
-
-    if (!xioctl(fd, VIDIOC_S_FMT, &format)
-            && format.fmt.pix.width == newwidth
-            && format.fmt.pix.height == newheight)
-    {
-        device->width = newwidth;
-        device->height = newheight;
-        device->swresize = FALSE;
-    }
-    else
-    {
-        TRACE("Using software resize: %dx%d -> %dx%d.\n",
-               format.fmt.pix.width, format.fmt.pix.height, device->width, device->height);
-        device->swresize = TRUE;
-    }
-    device->outputwidth = format.fmt.pix.width;
-    device->outputheight = format.fmt.pix.height;
     return S_OK;
 }
 
-HRESULT qcap_driver_get_format(const Capture *capBox, AM_MEDIA_TYPE ** mT)
+HRESULT qcap_driver_get_format(const Capture *device, AM_MEDIA_TYPE **mt)
 {
-    VIDEOINFOHEADER *vi;
+    *mt = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
+    if (!*mt)
+        return E_OUTOFMEMORY;
 
-    mT[0] = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
-    if (!mT[0])
-        return E_OUTOFMEMORY;
-    vi = CoTaskMemAlloc(sizeof(VIDEOINFOHEADER));
-    mT[0]->cbFormat = sizeof(VIDEOINFOHEADER);
-    if (!vi)
-    {
-        CoTaskMemFree(mT[0]);
-        mT[0] = NULL;
-        return E_OUTOFMEMORY;
-    }
-    mT[0]->majortype = MEDIATYPE_Video;
-    mT[0]->subtype = MEDIASUBTYPE_RGB24;
-    mT[0]->formattype = FORMAT_VideoInfo;
-    mT[0]->bFixedSizeSamples = TRUE;
-    mT[0]->bTemporalCompression = FALSE;
-    mT[0]->pUnk = NULL;
-    mT[0]->lSampleSize = capBox->outputwidth * capBox->outputheight * capBox->bitDepth / 8;
-    TRACE("Output format: %dx%d - %d bits = %u KB\n", capBox->outputwidth,
-          capBox->outputheight, capBox->bitDepth, mT[0]->lSampleSize/1024);
-    vi->rcSource.left = 0; vi->rcSource.top = 0;
-    vi->rcTarget.left = 0; vi->rcTarget.top = 0;
-    vi->rcSource.right = capBox->width; vi->rcSource.bottom = capBox->height;
-    vi->rcTarget.right = capBox->outputwidth; vi->rcTarget.bottom = capBox->outputheight;
-    vi->dwBitRate = capBox->fps * mT[0]->lSampleSize;
-    vi->dwBitErrorRate = 0;
-    vi->AvgTimePerFrame = (LONGLONG)10000000.0 / (LONGLONG)capBox->fps;
-    vi->bmiHeader.biSize = 40;
-    vi->bmiHeader.biWidth = capBox->outputwidth;
-    vi->bmiHeader.biHeight = capBox->outputheight;
-    vi->bmiHeader.biPlanes = 1;
-    vi->bmiHeader.biBitCount = 24;
-    vi->bmiHeader.biCompression = BI_RGB;
-    vi->bmiHeader.biSizeImage = mT[0]->lSampleSize;
-    vi->bmiHeader.biClrUsed = vi->bmiHeader.biClrImportant = 0;
-    vi->bmiHeader.biXPelsPerMeter = 100;
-    vi->bmiHeader.biYPelsPerMeter = 100;
-    mT[0]->pbFormat = (void *)vi;
-    return S_OK;
+    return CopyMediaType(*mt, &device->current_caps->media_type);
 }
 
 static __u32 v4l2_cid_from_qcap_property(VideoProcAmpProperty property)
@@ -323,59 +299,26 @@ HRESULT qcap_driver_set_prop(Capture *device, VideoProcAmpProperty property,
     return S_OK;
 }
 
-static void Resize(const Capture * capBox, LPBYTE output, const BYTE *input)
+static void reverse_image(const Capture *device, LPBYTE output, const BYTE *input)
 {
+    int inoffset, outoffset, pitch;
+    UINT width, height, depth;
+
+    width = device->current_caps->video_info.bmiHeader.biWidth;
+    height = device->current_caps->video_info.bmiHeader.biHeight;
+    depth = device->current_caps->video_info.bmiHeader.biBitCount / 8;
     /* the whole image needs to be reversed,
        because the dibs are messed up in windows */
-    if (!capBox->swresize)
+    outoffset = width * height * depth;
+    pitch = width * depth;
+    inoffset = 0;
+    while (outoffset > 0)
     {
-        int depth = capBox->bitDepth / 8;
-        int inoffset = 0, outoffset = capBox->height * capBox->width * depth;
-        int ow = capBox->width * depth;
-        while (outoffset > 0)
-        {
-            int x;
-            outoffset -= ow;
-            for (x = 0; x < ow; x++)
-                output[outoffset + x] = input[inoffset + x];
-            inoffset += ow;
-        }
-    }
-    else
-    {
-        HDC dc_s, dc_d;
-        HBITMAP bmp_s, bmp_d;
-        int depth = capBox->bitDepth / 8;
-        int inoffset = 0, outoffset = (capBox->outputheight) * capBox->outputwidth * depth;
-        int ow = capBox->outputwidth * depth;
-        LPBYTE myarray;
-
-        /* FIXME: Improve software resizing: add error checks and optimize */
-
-        myarray = CoTaskMemAlloc(capBox->outputwidth * capBox->outputheight * depth);
-        dc_s = CreateCompatibleDC(NULL);
-        dc_d = CreateCompatibleDC(NULL);
-        bmp_s = CreateBitmap(capBox->width, capBox->height, 1, capBox->bitDepth, input);
-        bmp_d = CreateBitmap(capBox->outputwidth, capBox->outputheight, 1, capBox->bitDepth, NULL);
-        SelectObject(dc_s, bmp_s);
-        SelectObject(dc_d, bmp_d);
-        StretchBlt(dc_d, 0, 0, capBox->outputwidth, capBox->outputheight,
-                   dc_s, 0, 0, capBox->width, capBox->height, SRCCOPY);
-        GetBitmapBits(bmp_d, capBox->outputwidth * capBox->outputheight * depth, myarray);
-        while (outoffset > 0)
-        {
-            int i;
-
-            outoffset -= ow;
-            for (i = 0; i < ow; i++)
-                output[outoffset + i] = myarray[inoffset + i];
-            inoffset += ow;
-        }
-        CoTaskMemFree(myarray);
-        DeleteObject(dc_s);
-        DeleteObject(dc_d);
-        DeleteObject(bmp_s);
-        DeleteObject(bmp_d);
+        int x;
+        outoffset -= pitch;
+        for (x = 0; x < pitch; x++)
+            output[outoffset + x] = input[inoffset + x];
+        inoffset += pitch;
     }
 }
 
@@ -387,8 +330,12 @@ static DWORD WINAPI ReadThread(LPVOID lParam)
     ULONG framecount = 0;
     unsigned char *pTarget, *image_data;
     unsigned int image_size;
+    UINT width, height, depth;
 
-    image_size = capBox->height * capBox->width * 3;
+    width = capBox->current_caps->video_info.bmiHeader.biWidth;
+    height = capBox->current_caps->video_info.bmiHeader.biHeight;
+    depth = capBox->current_caps->video_info.bmiHeader.biBitCount / 8;
+    image_size = width * height * depth;
     if (!(image_data = heap_alloc(image_size)))
     {
         ERR("Failed to allocate memory.\n");
@@ -405,10 +352,7 @@ static DWORD WINAPI ReadThread(LPVOID lParam)
         {
             int len;
             
-            if (!capBox->swresize)
-                len = capBox->height * capBox->width * capBox->bitDepth / 8;
-            else
-                len = capBox->outputheight * capBox->outputwidth * capBox->bitDepth / 8;
+            len = width * height * depth;
             IMediaSample_SetActualDataLength(pSample, len);
 
             len = IMediaSample_GetActualDataLength(pSample);
@@ -425,7 +369,7 @@ static DWORD WINAPI ReadThread(LPVOID lParam)
                 }
             }
 
-            Resize(capBox, pTarget, image_data);
+            reverse_image(capBox, pTarget, image_data);
             hr = IMemInputPin_Receive(capBox->pin->pMemInputPin, pSample);
             TRACE("%p -> Frame %u: %x\n", capBox, ++framecount, hr);
             IMediaSample_Release(pSample);
@@ -447,11 +391,8 @@ void qcap_driver_init_stream(Capture *device)
     HRESULT hr;
 
     req_props.cBuffers = 3;
-    if (!device->swresize)
-        req_props.cbBuffer = device->width * device->height;
-    else
-        req_props.cbBuffer = device->outputwidth * device->outputheight;
-    req_props.cbBuffer = (req_props.cbBuffer * device->bitDepth) / 8;
+    req_props.cbBuffer = device->current_caps->video_info.bmiHeader.biWidth * device->current_caps->video_info.bmiHeader.biHeight;
+    req_props.cbBuffer = (req_props.cbBuffer * device->current_caps->video_info.bmiHeader.biBitCount) / 8;
     req_props.cbAlign = 1;
     req_props.cbPrefix = 0;
 
@@ -495,18 +436,55 @@ void qcap_driver_cleanup_stream(Capture *device)
         ERR("Failed to decommit allocator, hr %#x.\n", hr);
 }
 
+
+static void fill_caps(__u32 pixelformat, __u32 width, __u32 height,
+        __u32 max_fps, __u32 min_fps, struct caps *caps)
+{
+    LONG depth = 24;
+
+    caps->video_info.dwBitRate = width * height * depth * max_fps;
+    caps->video_info.bmiHeader.biSize = sizeof(caps->video_info.bmiHeader);
+    caps->video_info.bmiHeader.biWidth = width;
+    caps->video_info.bmiHeader.biHeight = height;
+    caps->video_info.bmiHeader.biPlanes = 1;
+    caps->video_info.bmiHeader.biBitCount = depth;
+    caps->video_info.bmiHeader.biCompression = BI_RGB;
+    caps->video_info.bmiHeader.biSizeImage = width * height * depth / 8;
+    caps->media_type.majortype = MEDIATYPE_Video;
+    caps->media_type.subtype = MEDIASUBTYPE_RGB24;
+    caps->media_type.bFixedSizeSamples = TRUE;
+    caps->media_type.bTemporalCompression = FALSE;
+    caps->media_type.lSampleSize = width * height * depth / 8;
+    caps->media_type.formattype = FORMAT_VideoInfo;
+    caps->media_type.pUnk = NULL;
+    caps->media_type.cbFormat = sizeof(VIDEOINFOHEADER);
+    /* We reallocate the caps array, so pbFormat has to be set after all caps
+     * have been enumerated. */
+    caps->config.MaxFrameInterval = 10000000 * max_fps;
+    caps->config.MinFrameInterval = 10000000 * min_fps;
+    caps->config.MaxOutputSize.cx = width;
+    caps->config.MaxOutputSize.cy = height;
+    caps->config.MinOutputSize.cx = width;
+    caps->config.MinOutputSize.cy = height;
+    caps->config.guid = FORMAT_VideoInfo;
+    caps->config.MinBitsPerSecond = width * height * depth * min_fps;
+    caps->config.MaxBitsPerSecond = width * height * depth * max_fps;
+    caps->pixelformat = pixelformat;
+}
+
 Capture *qcap_driver_init(struct strmbase_source *pin, USHORT card)
 {
+    struct v4l2_frmsizeenum frmsize = {0};
     struct v4l2_capability caps = {{0}};
     struct v4l2_format format = {0};
     Capture *device = NULL;
     BOOL have_libv4l2;
     char path[20];
-    int fd;
+    int fd, i;
 
     have_libv4l2 = video_init();
 
-    if (!(device = CoTaskMemAlloc(sizeof(*device))))
+    if (!(device = heap_alloc_zero(sizeof(*device))))
         return NULL;
 
     sprintf(path, "/dev/video%i", card);
@@ -560,8 +538,70 @@ Capture *qcap_driver_init(struct strmbase_source *pin, USHORT card)
     }
 
     format.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR24;
-    if (xioctl(fd, VIDIOC_S_FMT, &format) == -1
+    if (xioctl(fd, VIDIOC_TRY_FMT, &format) == -1
             || format.fmt.pix.pixelformat != V4L2_PIX_FMT_BGR24)
+    {
+        ERR("This device doesn't support V4L2_PIX_FMT_BGR24 format.\n");
+        goto error;
+    }
+
+    frmsize.pixel_format = V4L2_PIX_FMT_BGR24;
+    while (xioctl(fd, VIDIOC_ENUM_FRAMESIZES, &frmsize) != -1)
+    {
+        struct v4l2_frmivalenum frmival = {0};
+        __u32 max_fps = 30, min_fps = 30;
+        struct caps *new_caps;
+
+        frmival.pixel_format = format.fmt.pix.pixelformat;
+        if (frmsize.type == V4L2_FRMSIZE_TYPE_DISCRETE)
+        {
+            frmival.width = frmsize.discrete.width;
+            frmival.height = frmsize.discrete.height;
+        }
+        else if (frmsize.type == V4L2_FRMSIZE_TYPE_STEPWISE)
+        {
+            frmival.width = frmsize.stepwise.max_width;
+            frmival.height = frmsize.stepwise.min_height;
+        }
+        else
+        {
+            FIXME("Unhandled frame size type: %d.\n", frmsize.type);
+            continue;
+        }
+
+        if (xioctl(fd, VIDIOC_ENUM_FRAMEINTERVALS, &frmival) != -1)
+        {
+            if (frmival.type == V4L2_FRMIVAL_TYPE_DISCRETE)
+            {
+                max_fps = frmival.discrete.denominator / frmival.discrete.numerator;
+                min_fps = max_fps;
+            }
+            else if (frmival.type == V4L2_FRMIVAL_TYPE_STEPWISE
+                    || frmival.type == V4L2_FRMIVAL_TYPE_CONTINUOUS)
+            {
+                max_fps = frmival.stepwise.max.denominator / frmival.stepwise.max.numerator;
+                min_fps = frmival.stepwise.min.denominator / frmival.stepwise.min.numerator;
+            }
+        }
+        else
+            ERR("Failed to get fps: %s.\n", strerror(errno));
+
+        new_caps = heap_realloc(device->caps, (device->caps_count + 1) * sizeof(*device->caps));
+        if (!new_caps)
+            goto error;
+        device->caps = new_caps;
+        fill_caps(format.fmt.pix.pixelformat, frmsize.discrete.width, frmsize.discrete.height,
+                max_fps, min_fps, &device->caps[device->caps_count]);
+        device->caps_count++;
+
+        frmsize.index++;
+    }
+
+    /* We reallocate the caps array, so we have to delay setting pbFormat. */
+    for (i = 0; i < device->caps_count; ++i)
+        device->caps[i].media_type.pbFormat = (BYTE *)&device->caps[i].video_info;
+
+    if (!set_caps(device, &device->caps[0]))
     {
         ERR("Failed to set pixel format: %s\n", strerror(errno));
         if (!have_libv4l2)
@@ -569,16 +609,13 @@ Capture *qcap_driver_init(struct strmbase_source *pin, USHORT card)
         goto error;
     }
 
-    device->outputwidth = device->width = format.fmt.pix.width;
-    device->outputheight = device->height = format.fmt.pix.height;
-    device->swresize = FALSE;
-    device->bitDepth = 24;
     device->pin = pin;
-    device->fps = 3;
     device->state = State_Stopped;
     device->run_event = CreateEventW(NULL, TRUE, FALSE, NULL);
 
-    TRACE("Format: %d bpp - %dx%d.\n", device->bitDepth, device->width, device->height);
+    TRACE("Format: %d bpp - %dx%d.\n", device->current_caps->video_info.bmiHeader.biBitCount,
+            device->current_caps->video_info.bmiHeader.biWidth,
+            device->current_caps->video_info.bmiHeader.biHeight);
 
     return device;
 
@@ -587,6 +624,26 @@ error:
     return NULL;
 }
 
+HRESULT qcap_driver_get_caps(Capture *device, LONG index, AM_MEDIA_TYPE **type,
+        VIDEO_STREAM_CONFIG_CAPS *vscc)
+{
+    if (index >= device->caps_count)
+        return S_FALSE;
+
+    *type = CreateMediaType(&device->caps[index].media_type);
+    if (!*type)
+        return E_OUTOFMEMORY;
+
+    if (vscc)
+        memcpy(vscc, &device->caps[index].config, sizeof(VIDEO_STREAM_CONFIG_CAPS));
+    return S_OK;
+}
+
+LONG qcap_driver_get_caps_count(Capture *device)
+{
+    return device->caps_count;
+}
+
 #else
 
 Capture *qcap_driver_init(struct strmbase_source *pin, USHORT card)
@@ -661,4 +718,16 @@ void qcap_driver_cleanup_stream(Capture *device)
     ERR("v4l absent: shouldn't be called\n");
 }
 
+HRESULT qcap_driver_get_caps(Capture *device, LONG index, AM_MEDIA_TYPE **type,
+        VIDEO_STREAM_CONFIG_CAPS *vscc)
+{
+    FAIL_WITH_ERR;
+}
+
+LONG qcap_driver_get_caps_count(Capture *device)
+{
+    ERR("v4l absent: shouldn't be called\n");
+    return 0;
+}
+
 #endif /* defined(VIDIOCMCAPTURE) */
