@@ -113,6 +113,7 @@ static HRESULT WINAPI enum_class_object_Next(
 {
     struct enum_class_object *ec = impl_from_IEnumWbemClassObject( iface );
     struct view *view = ec->query->view;
+    struct table *table;
     static int once = 0;
     HRESULT hr;
 
@@ -123,14 +124,15 @@ static HRESULT WINAPI enum_class_object_Next(
     if (lTimeout != WBEM_INFINITE && !once++) FIXME("timeout not supported\n");
 
     *puReturned = 0;
-    if (ec->index >= view->count) return WBEM_S_FALSE;
+    if (ec->index >= view->result_count) return WBEM_S_FALSE;
 
-    hr = create_class_object( view->table->name, iface, ec->index, NULL, apObjects );
+    table = get_view_table( view, ec->index );
+    hr = create_class_object( table->name, iface, ec->index, NULL, apObjects );
     if (hr != S_OK) return hr;
 
     ec->index++;
     *puReturned = 1;
-    if (ec->index == view->count && uCount > 1) return WBEM_S_FALSE;
+    if (ec->index == view->result_count && uCount > 1) return WBEM_S_FALSE;
     if (uCount > 1) return WBEM_S_TIMEDOUT;
     return WBEM_S_NO_ERROR;
 }
@@ -168,11 +170,11 @@ static HRESULT WINAPI enum_class_object_Skip(
 
     if (lTimeout != WBEM_INFINITE && !once++) FIXME("timeout not supported\n");
 
-    if (!view->count) return WBEM_S_FALSE;
+    if (!view->result_count) return WBEM_S_FALSE;
 
-    if (nCount > view->count - ec->index)
+    if (nCount > view->result_count - ec->index)
     {
-        ec->index = view->count - 1;
+        ec->index = view->result_count - 1;
         return WBEM_S_FALSE;
     }
     ec->index += nCount;
@@ -225,7 +227,6 @@ static struct record *create_record( struct table *table )
     for (i = 0; i < table->num_cols; i++)
     {
         record->fields[i].type    = table->columns[i].type;
-        record->fields[i].vartype = table->columns[i].vartype;
         record->fields[i].u.ival  = 0;
     }
     record->count = table->num_cols;
@@ -345,13 +346,13 @@ static HRESULT WINAPI class_object_GetQualifierSet(
 
 static HRESULT record_get_value( const struct record *record, UINT index, VARIANT *var, CIMTYPE *type )
 {
-    VARTYPE vartype = record->fields[index].vartype;
+    VARTYPE vartype = to_vartype( record->fields[index].type & CIM_TYPE_MASK );
 
     if (type) *type = record->fields[index].type;
 
     if (record->fields[index].type & CIM_FLAG_ARRAY)
     {
-        V_VT( var ) = vartype ? vartype : to_vartype( record->fields[index].type & CIM_TYPE_MASK ) | VT_ARRAY;
+        V_VT( var ) = vartype | VT_ARRAY;
         V_ARRAY( var ) = to_safearray( record->fields[index].u.aval, record->fields[index].type & CIM_TYPE_MASK );
         return S_OK;
     }
@@ -360,15 +361,12 @@ static HRESULT record_get_value( const struct record *record, UINT index, VARIAN
     case CIM_STRING:
     case CIM_DATETIME:
     case CIM_REFERENCE:
-        if (!vartype) vartype = VT_BSTR;
         V_BSTR( var ) = SysAllocString( record->fields[index].u.sval );
         break;
     case CIM_SINT32:
-        if (!vartype) vartype = VT_I4;
         V_I4( var ) = record->fields[index].u.ival;
         break;
     case CIM_UINT32:
-        if (!vartype) vartype = VT_UI4;
         V_UI4( var ) = record->fields[index].u.ival;
         break;
     default:
@@ -491,7 +489,7 @@ static HRESULT WINAPI class_object_GetNames(
     if (wszQualifierName || pQualifierVal)
         FIXME("qualifier not supported\n");
 
-    return get_properties( ec->query->view, lFlags, pNames );
+    return get_properties( ec->query->view, co->index, lFlags, pNames );
 }
 
 static HRESULT WINAPI class_object_BeginEnumeration(
@@ -519,17 +517,18 @@ static HRESULT WINAPI class_object_Next(
     struct class_object *obj = impl_from_IWbemClassObject( iface );
     struct enum_class_object *iter = impl_from_IEnumWbemClassObject( obj->iter );
     struct view *view = iter->query->view;
+    struct table *table = get_view_table( view, obj->index );
     BSTR prop;
     HRESULT hr;
     UINT i;
 
     TRACE("%p, %08x, %p, %p, %p, %p\n", iface, lFlags, strName, pVal, pType, plFlavor);
 
-    for (i = obj->index_property; i < view->table->num_cols; i++)
+    for (i = obj->index_property; i < table->num_cols; i++)
     {
-        if (is_method( view->table, i )) continue;
-        if (!is_selected_prop( view, view->table->columns[i].name )) continue;
-        if (!(prop = SysAllocString( view->table->columns[i].name ))) return E_OUTOFMEMORY;
+        if (is_method( table, i )) continue;
+        if (!is_result_prop( view, table->columns[i].name )) continue;
+        if (!(prop = SysAllocString( table->columns[i].name ))) return E_OUTOFMEMORY;
         if ((hr = get_propval( view, obj->index, prop, pVal, pType, plFlavor )) != S_OK)
         {
             SysFreeString( prop );
@@ -578,7 +577,6 @@ static HRESULT WINAPI class_object_Clone(
 
 static BSTR get_body_text( const struct table *table, UINT row, UINT *len )
 {
-    static const WCHAR fmtW[] = {'\n','\t','%','s',' ','=',' ','%','s',';',0};
     BSTR value, ret;
     WCHAR *p;
     UINT i;
@@ -588,7 +586,7 @@ static BSTR get_body_text( const struct table *table, UINT row, UINT *len )
     {
         if ((value = get_value_bstr( table, row, i )))
         {
-            *len += ARRAY_SIZE( fmtW );
+            *len += ARRAY_SIZE( L"\n\t%s = %s;" );
             *len += lstrlenW( table->columns[i].name );
             *len += SysStringLen( value );
             SysFreeString( value );
@@ -600,7 +598,7 @@ static BSTR get_body_text( const struct table *table, UINT row, UINT *len )
     {
         if ((value = get_value_bstr( table, row, i )))
         {
-            p += swprintf( p, *len - (p - ret), fmtW, table->columns[i].name, value );
+            p += swprintf( p, *len - (p - ret), L"\n\t%s = %s;", table->columns[i].name, value );
             SysFreeString( value );
         }
     }
@@ -609,18 +607,17 @@ static BSTR get_body_text( const struct table *table, UINT row, UINT *len )
 
 static BSTR get_object_text( const struct view *view, UINT index )
 {
-    static const WCHAR fmtW[] =
-        {'\n','i','n','s','t','a','n','c','e',' ','o','f',' ','%','s','\n','{','%','s','\n','}',';',0};
     UINT len, len_body, row = view->result[index];
+    struct table *table = get_view_table( view, index );
     BSTR ret, body;
 
-    len = ARRAY_SIZE( fmtW );
-    len += lstrlenW( view->table->name );
-    if (!(body = get_body_text( view->table, row, &len_body ))) return NULL;
+    len = ARRAY_SIZE( L"\ninstance of %s\n{%s\n};" );
+    len += lstrlenW( table->name );
+    if (!(body = get_body_text( table, row, &len_body ))) return NULL;
     len += len_body;
 
     if (!(ret = SysAllocStringLen( NULL, len ))) return NULL;
-    swprintf( ret, len, fmtW, view->table->name, body );
+    swprintf( ret, len, L"\ninstance of %s\n{%s\n};", table->name, body );
     SysFreeString( body );
     return ret;
 }
@@ -660,12 +657,12 @@ static HRESULT WINAPI class_object_SpawnInstance(
 {
     struct class_object *co = impl_from_IWbemClassObject( iface );
     struct enum_class_object *ec = impl_from_IEnumWbemClassObject( co->iter );
-    struct view *view = ec->query->view;
+    struct table *table = get_view_table( ec->query->view, co->index );
     struct record *record;
 
     TRACE("%p, %08x, %p\n", iface, lFlags, ppNewInstance);
 
-    if (!(record = create_record( view->table ))) return E_OUTOFMEMORY;
+    if (!(record = create_record( table ))) return E_OUTOFMEMORY;
 
     return create_class_object( co->name, NULL, 0, record, ppNewInstance );
 }
@@ -729,10 +726,6 @@ static void set_default_value( CIMTYPE type, UINT val, BYTE *ptr )
 static HRESULT create_signature_columns_and_data( IEnumWbemClassObject *iter, UINT *num_cols,
                                            struct column **cols, BYTE **data )
 {
-    static const WCHAR parameterW[] = {'P','a','r','a','m','e','t','e','r',0};
-    static const WCHAR typeW[] = {'T','y','p','e',0};
-    static const WCHAR varianttypeW[] = {'V','a','r','i','a','n','t','T','y','p','e',0};
-    static const WCHAR defaultvalueW[] = {'D','e','f','a','u','l','t','V','a','l','u','e',0};
     struct column *columns;
     BYTE *row;
     IWbemClassObject *param;
@@ -751,20 +744,16 @@ static HRESULT create_signature_columns_and_data( IEnumWbemClassObject *iter, UI
         IEnumWbemClassObject_Next( iter, WBEM_INFINITE, 1, &param, &count );
         if (!count) break;
 
-        hr = IWbemClassObject_Get( param, parameterW, 0, &val, NULL, NULL );
+        hr = IWbemClassObject_Get( param, L"Parameter", 0, &val, NULL, NULL );
         if (hr != S_OK) goto error;
         columns[i].name = heap_strdupW( V_BSTR( &val ) );
         VariantClear( &val );
 
-        hr = IWbemClassObject_Get( param, typeW, 0, &val, NULL, NULL );
+        hr = IWbemClassObject_Get( param, L"Type", 0, &val, NULL, NULL );
         if (hr != S_OK) goto error;
         columns[i].type = V_UI4( &val );
 
-        hr = IWbemClassObject_Get( param, varianttypeW, 0, &val, NULL, NULL );
-        if (hr != S_OK) goto error;
-        columns[i].vartype = V_UI4( &val );
-
-        hr = IWbemClassObject_Get( param, defaultvalueW, 0, &val, NULL, NULL );
+        hr = IWbemClassObject_Get( param, L"DefaultValue", 0, &val, NULL, NULL );
         if (hr != S_OK) goto error;
         if (V_UI4( &val )) set_default_value( columns[i].type, V_UI4( &val ), row + offset );
         offset += get_type_size( columns[i].type );
@@ -807,36 +796,26 @@ static HRESULT create_signature_table( IEnumWbemClassObject *iter, WCHAR *name )
 
 static WCHAR *build_signature_table_name( const WCHAR *class, const WCHAR *method, enum param_direction dir )
 {
-    static const WCHAR fmtW[] = {'_','_','%','s','_','%','s','_','%','s',0};
-    static const WCHAR outW[] = {'O','U','T',0};
-    static const WCHAR inW[] = {'I','N',0};
-    UINT len = ARRAY_SIZE(fmtW) + ARRAY_SIZE(outW) + lstrlenW( class ) + lstrlenW( method );
+    UINT len = ARRAY_SIZE(L"__%s_%s_%s") + ARRAY_SIZE(L"OUT") + lstrlenW( class ) + lstrlenW( method );
     WCHAR *ret;
 
     if (!(ret = heap_alloc( len * sizeof(WCHAR) ))) return NULL;
-    swprintf( ret, len, fmtW, class, method, dir == PARAM_IN ? inW : outW );
+    swprintf( ret, len, L"__%s_%s_%s", class, method, dir == PARAM_IN ? L"IN" : L"OUT" );
     return wcsupr( ret );
 }
 
 HRESULT create_signature( const WCHAR *class, const WCHAR *method, enum param_direction dir,
                           IWbemClassObject **sig )
 {
-    static const WCHAR selectW[] =
-        {'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
-         '_','_','P','A','R','A','M','E','T','E','R','S',' ','W','H','E','R','E',' ',
-         'C','l','a','s','s','=','\'','%','s','\'',' ','A','N','D',' ',
-         'M','e','t','h','o','d','=','\'','%','s','\'',' ','A','N','D',' ',
-         'D','i','r','e','c','t','i','o','n','%','s',0};
-    static const WCHAR geW[] = {'>','=','0',0};
-    static const WCHAR leW[] = {'<','=','0',0};
-    UINT len = ARRAY_SIZE(selectW) + ARRAY_SIZE(geW);
+    static const WCHAR selectW[] = L"SELECT * FROM __PARAMETERS WHERE Class='%s' AND Method='%s' AND Direction%s";
+    UINT len = ARRAY_SIZE(selectW) + ARRAY_SIZE(L">=0");
     IEnumWbemClassObject *iter;
     WCHAR *query, *name;
     HRESULT hr;
 
     len += lstrlenW( class ) + lstrlenW( method );
     if (!(query = heap_alloc( len * sizeof(WCHAR) ))) return E_OUTOFMEMORY;
-    swprintf( query, len, selectW, class, method, dir >= 0 ? geW : leW );
+    swprintf( query, len, selectW, class, method, dir >= 0 ? L">=0" : L"<=0" );
 
     hr = exec_query( query, &iter );
     heap_free( query );
