@@ -48,6 +48,9 @@
 
 int needs_get_pc_thunk = 0;
 
+static const char builtin_signature[32] = "Wine builtin DLL";
+static const char fakedll_signature[32] = "Wine placeholder DLL";
+
 /* check if entry point needs a relay thunk */
 static inline int needs_relay( const ORDDEF *odp )
 {
@@ -379,9 +382,11 @@ static void output_relay_debug( DLLSPEC *spec )
 void output_exports( DLLSPEC *spec )
 {
     int i, fwd_size = 0;
+    int needs_imports = 0;
     int needs_relay = has_relays( spec );
     int nr_exports = spec->base <= spec->limit ? spec->limit - spec->base + 1 : 0;
     const char *func_ptr = (target_platform == PLATFORM_WINDOWS) ? ".rva" : get_asm_ptr_keyword();
+    const char *name;
 
     if (!nr_exports) return;
 
@@ -430,6 +435,13 @@ void output_exports( DLLSPEC *spec )
                 output( "\t%s .L__wine_spec_forwards+%u\n", func_ptr, fwd_size );
                 fwd_size += strlen(odp->link_name) + 1;
             }
+            else if ((odp->flags & FLAG_IMPORT) && (target_cpu == CPU_x86 || target_cpu == CPU_x86_64))
+            {
+                name = odp->name ? odp->name : odp->export_name;
+                if (name) output( "\t%s %s_%s\n", func_ptr, asm_name("__wine_spec_imp"), name );
+                else output( "\t%s %s_%u\n", func_ptr, asm_name("__wine_spec_imp"), i );
+                needs_imports = 1;
+            }
             else if (odp->flags & FLAG_EXT_LINK)
             {
                 output( "\t%s %s_%s\n", func_ptr, asm_name("__wine_spec_ext_link"), odp->link_name );
@@ -503,68 +515,74 @@ void output_exports( DLLSPEC *spec )
 
     /* output relays */
 
-    if (target_platform == PLATFORM_WINDOWS)
+    if (needs_relay)
     {
-        if (!needs_relay) return;
-        output( "\t.data\n" );
-        output( "\t.align %d\n", get_alignment(get_ptr_size()) );
-    }
-    else
-    {
-        output( "\t.align %d\n", get_alignment(get_ptr_size()) );
-        output( ".L__wine_spec_exports_end:\n" );
-        if (!needs_relay)
+        if (target_platform == PLATFORM_WINDOWS)
         {
-            output( "\t%s 0\n", get_asm_ptr_keyword() );
-            return;
+            output( "\t.data\n" );
+            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
         }
+        else
+        {
+            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+            output( ".L__wine_spec_exports_end:\n" );
+        }
+
+        output( ".L__wine_spec_relay_descr:\n" );
+        output( "\t%s 0xdeb90002\n", get_asm_ptr_keyword() );  /* magic */
+        output( "\t%s 0\n", get_asm_ptr_keyword() );           /* relay func */
+        output( "\t%s 0\n", get_asm_ptr_keyword() );           /* private data */
+        output( "\t%s __wine_spec_relay_entry_points\n", get_asm_ptr_keyword() );
+        output( "\t%s .L__wine_spec_relay_entry_point_offsets\n", get_asm_ptr_keyword() );
+        output( "\t%s .L__wine_spec_relay_args_string\n", get_asm_ptr_keyword() );
+
+        output_relay_debug( spec );
+    }
+    else if (target_platform != PLATFORM_WINDOWS)
+    {
+            output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+            output( ".L__wine_spec_exports_end:\n" );
+            output( "\t%s 0\n", get_asm_ptr_keyword() );
     }
 
-    output( ".L__wine_spec_relay_descr:\n" );
-    output( "\t%s 0xdeb90002\n", get_asm_ptr_keyword() );  /* magic */
-    output( "\t%s 0\n", get_asm_ptr_keyword() );           /* relay func */
-    output( "\t%s 0\n", get_asm_ptr_keyword() );           /* private data */
-    output( "\t%s __wine_spec_relay_entry_points\n", get_asm_ptr_keyword() );
-    output( "\t%s .L__wine_spec_relay_entry_point_offsets\n", get_asm_ptr_keyword() );
-    output( "\t%s .L__wine_spec_relay_args_string\n", get_asm_ptr_keyword() );
+    /* output import thunks */
 
-    output_relay_debug( spec );
-}
-
-
-/*******************************************************************
- *         output_asm_constructor
- *
- * Output code for calling a dll constructor.
- */
-static void output_asm_constructor( const char *constructor )
-{
-    if (target_platform == PLATFORM_APPLE)
+    if (!needs_imports) return;
+    output( "\t.text\n" );
+    for (i = spec->base; i <= spec->limit; i++)
     {
-        /* Mach-O doesn't have an init section */
-        output( "\n\t.mod_init_func\n" );
-        output( "\t.align %d\n", get_alignment(get_ptr_size()) );
-        output( "\t%s %s\n", get_asm_ptr_keyword(), asm_name(constructor) );
-    }
-    else
-    {
-        switch(target_cpu)
+        ORDDEF *odp = spec->ordinals[i];
+        if (!odp) continue;
+        if (!(odp->flags & FLAG_IMPORT)) continue;
+
+        name = odp->name ? odp->name : odp->export_name;
+
+        output( "\t.align %d\n", get_alignment(4) );
+        output( "\t.long 0x90909090,0x90909090\n" );
+        if (name) output( "%s_%s:\n", asm_name("__wine_spec_imp"), name );
+        else output( "%s_%u:\n", asm_name("__wine_spec_imp"), i );
+        output_cfi( ".cfi_startproc" );
+
+        switch (target_cpu)
         {
         case CPU_x86:
+            output( "\t.byte 0x8b,0xff,0x55,0x8b,0xec,0x5d\n" );  /* hotpatch prolog */
+            if (UsePIC)
+            {
+                output( "\tcall %s\n", asm_name("__wine_spec_get_pc_thunk_eax") );
+                output( "1:\tjmp *__imp_%s-1b(%%eax)\n", asm_name( get_link_name( odp )));
+                needs_get_pc_thunk = 1;
+            }
+            else output( "\tjmp *__imp_%s\n", asm_name( get_link_name( odp )));
+            break;
         case CPU_x86_64:
-            output( "\n\t.section \".init\",\"ax\"\n" );
-            output( "\tcall %s\n", asm_name(constructor) );
-            break;
-        case CPU_ARM:
-            output( "\n\t.section \".text\",\"ax\"\n" );
-            output( "\tblx %s\n", asm_name(constructor) );
-            break;
-        case CPU_ARM64:
-        case CPU_POWERPC:
-            output( "\n\t.section \".init\",\"ax\"\n" );
-            output( "\tbl %s\n", asm_name(constructor) );
+            output( "\t.byte 0x48\n" );  /* hotpatch prolog */
+            output( "\tjmp *__imp_%s(%%rip)\n", asm_name( get_link_name( odp )));
             break;
+        default:
+            assert(0);
         }
+        output_cfi( ".cfi_endproc" );
     }
 }
 
@@ -625,7 +643,8 @@ void output_module( DLLSPEC *spec )
 
     output( "\n\t.data\n" );
     output( "\t.align %d\n", get_alignment(get_ptr_size()) );
-    output( "%s\n", asm_globl("__wine_spec_nt_header") );
+    output( "\t.globl %s\n", asm_name("__wine_spec_nt_header") );
+    output( "%s:\n", asm_name("__wine_spec_nt_header") );
     output( ".L__wine_spec_rva_base:\n" );
 
     output( "\t.long 0x4550\n" );         /* Signature */
@@ -694,13 +713,8 @@ void output_module( DLLSPEC *spec )
 
     output_data_directories( data_dirs );
 
-    output( "\n\t%s\n", get_asm_string_section() );
-    output( "%s\n", asm_globl("__wine_spec_file_name") );
-    output( "\t%s \"%s\"\n", get_asm_string_keyword(), spec->file_name );
     if (target_platform == PLATFORM_APPLE)
         output( "\t.lcomm %s,4\n", asm_name("_end") );
-
-    output_asm_constructor( "__wine_spec_init_ctor" );
 }
 
 
@@ -738,12 +752,11 @@ void output_fake_module( DLLSPEC *spec )
     static const unsigned char exe_code_section[] = { 0xb8, 0x01, 0x00, 0x00, 0x00,  /* movl $1,%eax */
                                                       0xc2, 0x04, 0x00 };            /* ret $4 */
 
-    static const char fakedll_signature[] = "Wine placeholder DLL";
     const unsigned int page_size = get_page_size();
     const unsigned int section_align = page_size;
     const unsigned int file_align = 0x200;
     const unsigned int reloc_size = 8;
-    const unsigned int lfanew = (0x40 + sizeof(fakedll_signature) + 15) & ~15;
+    const unsigned int lfanew = 0x40 + sizeof(fakedll_signature);
     const unsigned int nb_sections = 2 + (spec->nb_resources != 0);
     const unsigned int text_size = (spec->characteristics & IMAGE_FILE_DLL) ?
                                     sizeof(dll_code_section) : sizeof(exe_code_section);
@@ -785,7 +798,6 @@ void output_fake_module( DLLSPEC *spec )
     put_dword( lfanew );
 
     put_data( fakedll_signature, sizeof(fakedll_signature) );
-    align_output( 16 );
 
     put_dword( 0x4550 );                             /* Signature */
     switch(target_cpu)
@@ -931,7 +943,7 @@ void output_fake_module( DLLSPEC *spec )
  *
  * Build a Win32 def file from a spec file.
  */
-void output_def_file( DLLSPEC *spec, int include_stubs )
+void output_def_file( DLLSPEC *spec, int import_only )
 {
     DLLSPEC *spec32 = NULL;
     const char *name;
@@ -965,7 +977,7 @@ void output_def_file( DLLSPEC *spec, int include_stubs )
         else continue;
 
         if (!is_private) total++;
-        if (!include_stubs && odp->type == TYPE_STUB) continue;
+        if (import_only && odp->type == TYPE_STUB) continue;
 
         if ((odp->flags & FLAG_FASTCALL) && target_platform == PLATFORM_WINDOWS)
             name = strmake( "@%s", name );
@@ -980,13 +992,14 @@ void output_def_file( DLLSPEC *spec, int include_stubs )
         case TYPE_VARARGS:
         case TYPE_CDECL:
             /* try to reduce output */
-            if(strcmp(name, odp->link_name) || (odp->flags & FLAG_FORWARD))
+            if(!import_only && (strcmp(name, odp->link_name) || (odp->flags & FLAG_FORWARD)))
                 output( "=%s", odp->link_name );
             break;
         case TYPE_STDCALL:
         {
             int at_param = get_args_size( odp );
             if (!kill_at && target_cpu == CPU_x86) output( "@%d", at_param );
+            if (import_only) break;
             if  (odp->flags & FLAG_FORWARD)
                 output( "=%s", odp->link_name );
             else if (strcmp(name, odp->link_name)) /* try to reduce output */
@@ -1009,3 +1022,190 @@ void output_def_file( DLLSPEC *spec, int include_stubs )
     if (!total) warning( "%s: Import library doesn't export anything\n", spec->file_name );
     if (spec32) free_dll_spec( spec32 );
 }
+
+
+/*******************************************************************
+ *         make_builtin_files
+ */
+void make_builtin_files( char *argv[] )
+{
+    int i, fd;
+    struct
+    {
+        unsigned short e_magic;
+        unsigned short unused[29];
+        unsigned int   e_lfanew;
+    } header;
+
+    for (i = 0; argv[i]; i++)
+    {
+        if ((fd = open( argv[i], O_RDWR | O_BINARY )) == -1) fatal_perror( "Cannot open %s", argv[i] );
+        if (read( fd, &header, sizeof(header) ) == sizeof(header) && !memcmp( &header.e_magic, "MZ", 2 ))
+        {
+            if (header.e_lfanew < sizeof(header) + sizeof(builtin_signature))
+                fatal_error( "%s: Not enough space (%x) for Wine signature\n", argv[i], header.e_lfanew );
+            write( fd, builtin_signature, sizeof(builtin_signature) );
+        }
+        else fatal_error( "%s: Unrecognized file format\n", argv[i] );
+        close( fd );
+    }
+}
+
+static void fixup_elf32( const char *name, int fd, void *header, size_t header_size )
+{
+    struct
+    {
+        unsigned char  e_ident[16];
+        unsigned short e_type;
+        unsigned short e_machine;
+        unsigned int   e_version;
+        unsigned int   e_entry;
+        unsigned int   e_phoff;
+        unsigned int   e_shoff;
+        unsigned int   e_flags;
+        unsigned short e_ehsize;
+        unsigned short e_phentsize;
+        unsigned short e_phnum;
+        unsigned short e_shentsize;
+        unsigned short e_shnum;
+        unsigned short e_shstrndx;
+    } *elf = header;
+    struct
+    {
+        unsigned int p_type;
+        unsigned int p_offset;
+        unsigned int p_vaddr;
+        unsigned int p_paddr;
+        unsigned int p_filesz;
+        unsigned int p_memsz;
+        unsigned int p_flags;
+        unsigned int p_align;
+    } *phdr;
+    struct
+    {
+        unsigned int d_tag;
+        unsigned int d_val;
+    } *dyn;
+
+    unsigned int i, size;
+
+    if (header_size < sizeof(*elf)) return;
+    if (elf->e_ident[6] != 1 /* EV_CURRENT */) return;
+
+    size = elf->e_phnum * elf->e_phentsize;
+    phdr = xmalloc( size );
+    lseek( fd, elf->e_phoff, SEEK_SET );
+    if (read( fd, phdr, size ) != size) return;
+
+    for (i = 0; i < elf->e_phnum; i++)
+    {
+        if (phdr->p_type == 2 /* PT_DYNAMIC */ ) break;
+        phdr = (void *)((char *)phdr + elf->e_phentsize);
+    }
+    if (i == elf->e_phnum) return;
+
+    dyn = xmalloc( phdr->p_filesz );
+    lseek( fd, phdr->p_offset, SEEK_SET );
+    if (read( fd, dyn, phdr->p_filesz ) != phdr->p_filesz) return;
+    for (i = 0; i < phdr->p_filesz / sizeof(*dyn) && dyn[i].d_tag; i++)
+    {
+        switch (dyn[i].d_tag)
+        {
+        case 25: dyn[i].d_tag = 0x60009990; break;  /* DT_INIT_ARRAY */
+        case 27: dyn[i].d_tag = 0x60009991; break;  /* DT_INIT_ARRAYSZ */
+        case 12: dyn[i].d_tag = 0x60009992; break;  /* DT_INIT */
+        }
+    }
+    lseek( fd, phdr->p_offset, SEEK_SET );
+    write( fd, dyn, phdr->p_filesz );
+}
+
+static void fixup_elf64( const char *name, int fd, void *header, size_t header_size )
+{
+    struct
+    {
+        unsigned char    e_ident[16];
+        unsigned short   e_type;
+        unsigned short   e_machine;
+        unsigned int     e_version;
+        unsigned __int64 e_entry;
+        unsigned __int64 e_phoff;
+        unsigned __int64 e_shoff;
+        unsigned int     e_flags;
+        unsigned short   e_ehsize;
+        unsigned short   e_phentsize;
+        unsigned short   e_phnum;
+        unsigned short   e_shentsize;
+        unsigned short   e_shnum;
+        unsigned short   e_shstrndx;
+    } *elf = header;
+    struct
+    {
+        unsigned int     p_type;
+        unsigned int     p_flags;
+        unsigned __int64 p_offset;
+        unsigned __int64 p_vaddr;
+        unsigned __int64 p_paddr;
+        unsigned __int64 p_filesz;
+        unsigned __int64 p_memsz;
+        unsigned __int64 p_align;
+    } *phdr;
+    struct
+    {
+        unsigned __int64 d_tag;
+        unsigned __int64 d_val;
+    } *dyn;
+
+    unsigned int i, size;
+
+    if (header_size < sizeof(*elf)) return;
+    if (elf->e_ident[6] != 1 /* EV_CURRENT */) return;
+
+    size = elf->e_phnum * elf->e_phentsize;
+    phdr = xmalloc( size );
+    lseek( fd, elf->e_phoff, SEEK_SET );
+    if (read( fd, phdr, size ) != size) return;
+
+    for (i = 0; i < elf->e_phnum; i++)
+    {
+        if (phdr->p_type == 2 /* PT_DYNAMIC */ ) break;
+        phdr = (void *)((char *)phdr + elf->e_phentsize);
+    }
+    if (i == elf->e_phnum) return;
+
+    dyn = xmalloc( phdr->p_filesz );
+    lseek( fd, phdr->p_offset, SEEK_SET );
+    if (read( fd, dyn, phdr->p_filesz ) != phdr->p_filesz) return;
+    for (i = 0; i < phdr->p_filesz / sizeof(*dyn) && dyn[i].d_tag; i++)
+    {
+        switch (dyn[i].d_tag)
+        {
+        case 25: dyn[i].d_tag = 0x60009990; break;  /* DT_INIT_ARRAY */
+        case 27: dyn[i].d_tag = 0x60009991; break;  /* DT_INIT_ARRAYSZ */
+        case 12: dyn[i].d_tag = 0x60009992; break;  /* DT_INIT */
+        }
+    }
+    lseek( fd, phdr->p_offset, SEEK_SET );
+    write( fd, dyn, phdr->p_filesz );
+}
+
+/*******************************************************************
+ *         fixup_constructors
+ */
+void fixup_constructors( char *argv[] )
+{
+    int i, fd, size;
+    unsigned int header[64];
+
+    for (i = 0; argv[i]; i++)
+    {
+        if ((fd = open( argv[i], O_RDWR | O_BINARY )) == -1) fatal_perror( "Cannot open %s", argv[i] );
+        size = read( fd, &header, sizeof(header) );
+        if (size > 5)
+        {
+            if (!memcmp( header, "\177ELF\001", 5 )) fixup_elf32( argv[i], fd, header, size );
+            else if (!memcmp( header, "\177ELF\002", 5 )) fixup_elf64( argv[i], fd, header, size );
+        }
+        close( fd );
+    }
+}
